use BDD::Behave;

describe 'file-line-shorthand fixture', :order<defined>, {
  it 'first example',  { expect(1).to.eq(1) }
  it 'second example', { expect(2).to.eq(2) }
  it 'third example',  { expect(3).to.eq(3) }
}
